# -*- coding: utf-8 -*-

"""Workouts class"""

from sqlobject import *
from workoutsql import *
from Hercules.workoutsql import *
from Hercules.globals import herculesDBFile


class Workouts:
    """A class representing a workouts essence"""

    def __init__(self):
        """set up connection to database"""
        connectionString = 'sqlite:' + herculesDBFile
        connection = connectionForURI(connectionString)
        WorkoutSQL._connection = connection
        SetSQL._connection = connection

        WorkoutSQL.createTable(ifNotExists=True)
        SetSQL.createTable(ifNotExists=True)

    def SelectWorkout(self, workoutDateID):
        """Return instances of all exercises"""
        workout = WorkoutSQL.select(WorkoutSQL.q.workoutDate==workoutDateID)
        try:
            p = workout[0]
        except IndexError:
            return [], []
        sets = SetSQL.select(AND(SetSQL.q.workoutID==p.id))
        return workout, sets
        
    def GetWorkout(self, workoutDateID):
        """Get workout instance"""
        workout = WorkoutSQL.select(WorkoutSQL.q.workoutDate==workoutDateID)
        try:
            p = workout[0]
        except IndexError:
            return []
        return WorkoutSQL.get(p.id)

    def GetSet(self, num):
        """Get set instance"""
        return SetSQL.get(num)
        
    def InsertWorkout(self, date, startTime, finishTime):
        """Insert workout info into database"""
        return WorkoutSQL(workoutDate=date,
                          workoutStartTime=startTime,
                          workoutFinishTime=finishTime)

    def InsertSet(self, title, setType, setNum, setReps, setWeight, workout):
        """Insert set into workout"""
        return SetSQL(setTitle=title, setType=setType, setNum=setNum,
                      setReps=setReps, setWeight=setWeight, workout=workout)

    def UpdateWorkout(self, oldWorkout, startTime, finishTime):
        """Update workout instance"""
        oldWorkout.workoutStartTime = startTime
        oldWorkout.workoutFinishTime = finishTime

    def UpdateSet(self, oldSet, title, setType, setNum, setReps, setWeight):
        """Update set instance"""
        oldSet.setTitle = title
        oldSet.setType = setType
        oldSet.setNum = setNum
        oldSet.setReps = setReps
        oldSet.setWeight = setWeight

    def DeleteWorkout(self, workout):
        """Delete workout instance"""
        WorkoutSQL.delete(workout.id)

    def DeleteWorkoutWithSets(self, workout):
        """Delete workout with sets"""
        sets = SetSQL.select(SetSQL.q.workoutID==workout.id)
        for p in sets:
            self.DeleteSet(p)
        self.DeleteWorkout(workout)

    def DeleteSet(self, oldSet):
        """Delete set instance"""
        SetSQL.delete(oldSet.id)
